import 'package:flutter/widgets.dart';
import 'package:flutterskel/configuration/ConfigurationRepository.dart';
import 'package:flutterskel/datasource/DataSource.dart';
import 'package:flutterskel/resources/images/ImageResources.dart';
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart';
import 'package:flutterskel/routing/page/PageRouter.dart';
import 'package:flutterskel/routing/url/UrlRouter.dart';
import 'package:flutterskel/userinformation/UserInformationRepository.dart';
import 'package:flutterskel/resources/icons/IconResources.dart';
import 'package:provider/provider.dart';

class ProviderBuilder<Data extends DataSource,
                      URouter extends UrlRouter,
                      PRouter extends PageRouter,
                      Images extends ImageResources,
                      ConfigRepo extends ConfigurationRepository,
                      UserRepo extends UserInformationRepository,
                      StringRepo extends StringsRepository,
                      ThemeRepo extends ThemeRepository,
                      Icons extends IconResources> {

  Data dataSource;
  ConfigRepo configuration;

  URouter urlRouter;
  PRouter pageRouter;

  Images images;

  UserRepo userInfoRepo;
  StringRepo stringRepo;
  ThemeRepo themeRepo;

  Icons icons;

  ProviderBuilder({
    this.dataSource,
    this.configuration,
    this.userInfoRepo,
    this.urlRouter,
    this.pageRouter,
    this.images,
    this.stringRepo,
    this.themeRepo,
    this.icons
  });

  MultiProvider build({@required Widget child}) {
    return MultiProvider(
      providers: [
        Provider.value(value: dataSource),
        Provider.value(value: configuration),
        Provider.value(value: urlRouter),
        Provider.value(value: pageRouter),
        Provider.value(value: images),
        Provider.value(value: icons),

        // Change Notifiers
        ChangeNotifierProvider.value(value: userInfoRepo),
        ChangeNotifierProvider.value(value: stringRepo),
        ChangeNotifierProvider.value(value: themeRepo)
      ],
      child: child,
    );
  }

}
