
import 'package:flutter/material.dart';

import '../colors/ColorResources.dart';

/// Base class for fonts
/// Provides basic Material Design textStyles
abstract class FontResources {

  ColorResources colors;
  
  // Font Family
  String fontFamily;

  // Font Sizes
  final double sizeHeadline1;
  final double sizeHeadline2;
  final double sizeHeadline3;
  final double sizeHeadline4;
  final double sizeHeadline5;
  final double sizeHeadline6;
  final double sizeSectionTitle;
  final double sizeSubtitle1;
  final double sizeSubtitle2;
  final double sizeBody1;
  final double sizeBody2;
  final double sizeButton;
  final double sizeCaption;
  final double sizeOverline;

  // Font Spacings
  final double letterSpacingHeadline1;
  final double letterSpacingHeadline2;
  final double letterSpacingHeadline3;
  final double letterSpacingHeadline4;
  final double letterSpacingHeadline5;
  final double letterSpacingHeadline6;
  final double letterSpacingSectionTitle;
  final double letterSpacingSubtitle1;
  final double letterSpacingSubtitle2;
  final double letterSpacingBody1;
  final double letterSpacingBody2;
  final double letterSpacingButton;
  final double letterSpacingCaption;
  final double letterSpacingOverline;
  
  // Font Styles
  FontStyle fontStyleHeadline1;
  FontStyle fontStyleHeadline2;
  FontStyle fontStyleHeadline3;
  FontStyle fontStyleHeadline4;
  FontStyle fontStyleHeadline5;
  FontStyle fontStyleHeadline6;
  FontStyle fontStyleSectionTitle;
  FontStyle fontStyleSubtitle1;
  FontStyle fontStyleSubtitle2;
  FontStyle fontStyleBody1;
  FontStyle fontStyleBody2;
  FontStyle fontStyleButton;
  FontStyle fontStyleCaption;
  FontStyle fontStyleOverline;

  // Font Weights
  FontWeight fontWeightHeadline1;
  FontWeight fontWeightHeadline2;
  FontWeight fontWeightHeadline3;
  FontWeight fontWeightHeadline4;
  FontWeight fontWeightHeadline5;
  FontWeight fontWeightHeadline6;
  FontWeight fontWeightSectionTitle;
  FontWeight fontWeightSubtitle1;
  FontWeight fontWeightSubtitle2;
  FontWeight fontWeightBody1;
  FontWeight fontWeightBody2;
  FontWeight fontWeightButton;
  FontWeight fontWeightCaption;
  FontWeight fontWeightOverline;

  // Text Styles
  TextStyle styleHeadline1;
  TextStyle styleHeadline2;
  TextStyle styleHeadline3;
  TextStyle styleHeadline4;
  TextStyle styleHeadline5;
  TextStyle styleHeadline6;
  TextStyle styleSectionTitle;
  TextStyle styleSubtitle1;
  TextStyle styleSubtitle2;
  TextStyle styleBody1;
  TextStyle styleBody2;
  TextStyle styleButton;
  TextStyle styleCaption;
  TextStyle styleOverline;
  
  FontResources({
    // Font Family
    this.fontFamily,
    // Sizes
    this.sizeHeadline1 = 96,
    this.sizeHeadline2 = 60,
    this.sizeHeadline3 = 48,
    this.sizeHeadline4 = 34,
    this.sizeHeadline5 = 24,
    this.sizeHeadline6 = 20,
    this.sizeSectionTitle = 22,
    this.sizeSubtitle1 = 16,
    this.sizeSubtitle2 = 14,
    this.sizeBody1 = 16,
    this.sizeBody2 = 14,
    this.sizeButton = 14,
    this.sizeCaption = 12,
    this.sizeOverline = 10,
    // Spacing
    this.letterSpacingHeadline1 = -1.5,
    this.letterSpacingHeadline2 = -0.5,
    this.letterSpacingHeadline3 = 0,
    this.letterSpacingHeadline4 = 0.25,
    this.letterSpacingHeadline5 = 0,
    this.letterSpacingHeadline6 = 0.15,
    this.letterSpacingSectionTitle = 0.5,
    this.letterSpacingSubtitle1 = 0.15,
    this.letterSpacingSubtitle2 = 0.1,
    this.letterSpacingBody1 = 0.5,
    this.letterSpacingBody2 = 0.25,
    this.letterSpacingButton = 1.25,
    this.letterSpacingCaption = 0.4,
    this.letterSpacingOverline = 1.5,
    // Font Style
    this.fontStyleHeadline1 = FontStyle.normal,
    this.fontStyleHeadline2 = FontStyle.normal,
    this.fontStyleHeadline3 = FontStyle.normal,
    this.fontStyleHeadline4 = FontStyle.normal,
    this.fontStyleHeadline5 = FontStyle.normal,
    this.fontStyleHeadline6 = FontStyle.normal,
    this.fontStyleSectionTitle = FontStyle.normal,
    this.fontStyleSubtitle1 = FontStyle.normal,
    this.fontStyleSubtitle2 = FontStyle.normal,
    this.fontStyleBody1 = FontStyle.normal,
    this.fontStyleBody2 = FontStyle.normal,
    this.fontStyleButton = FontStyle.normal,
    this.fontStyleCaption = FontStyle.normal,
    this.fontStyleOverline = FontStyle.normal,
    // Font Weight
    this.fontWeightHeadline1 = FontWeight.w600,
    this.fontWeightHeadline2 = FontWeight.w400,
    this.fontWeightHeadline3 = FontWeight.w400,
    this.fontWeightHeadline4 = FontWeight.w400,
    this.fontWeightHeadline5 = FontWeight.w400,
    this.fontWeightHeadline6 = FontWeight.w400,
    this.fontWeightSectionTitle = FontWeight.w400,
    this.fontWeightSubtitle1 = FontWeight.w400,
    this.fontWeightSubtitle2 = FontWeight.w400,
    this.fontWeightBody1 = FontWeight.w400,
    this.fontWeightBody2 = FontWeight.w400,
    this.fontWeightButton = FontWeight.w400,
    this.fontWeightCaption = FontWeight.w400,
    this.fontWeightOverline = FontWeight.w500,
    // Colors
    @required this.colors,
  }) {
    
    TextStyle baseStyle = TextStyle();
    if(fontFamily != null) {
      baseStyle = TextStyle(fontFamily: this.fontFamily);
    } else {
      baseStyle = TextStyle();
    }
    
    styleHeadline1 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightHeadline1, fontStyle: fontStyleHeadline1, letterSpacing: letterSpacingHeadline1, fontSize: sizeHeadline1);
    styleHeadline2 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightHeadline2, fontStyle: fontStyleHeadline2, letterSpacing: letterSpacingHeadline2, fontSize: sizeHeadline2);
    styleHeadline3 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightHeadline3, fontStyle: fontStyleHeadline3, letterSpacing: letterSpacingHeadline3, fontSize: sizeHeadline3);
    styleHeadline4 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightHeadline4, fontStyle: fontStyleHeadline4, letterSpacing: letterSpacingHeadline4, fontSize: sizeHeadline4);
    styleHeadline5 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightHeadline5, fontStyle: fontStyleHeadline5, letterSpacing: letterSpacingHeadline5, fontSize: sizeHeadline5);
    styleHeadline6 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightHeadline6, fontStyle: fontStyleHeadline6, letterSpacing: letterSpacingHeadline6, fontSize: sizeHeadline6);

    styleSectionTitle = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightSectionTitle, fontStyle: fontStyleSectionTitle, letterSpacing: letterSpacingSectionTitle, fontSize: sizeSectionTitle);
    
    styleSubtitle1 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightSubtitle1, fontStyle: fontStyleSubtitle1, letterSpacing: letterSpacingSubtitle1, fontSize: sizeSubtitle1);
    styleSubtitle2 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightSubtitle2, fontStyle: fontStyleSubtitle2, letterSpacing: letterSpacingSubtitle2, fontSize: sizeSubtitle2);

    styleBody1 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightBody1, fontStyle: fontStyleBody1, letterSpacing: letterSpacingBody1, fontSize: sizeBody1);
    styleBody2 = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightBody2, fontStyle: fontStyleBody2, letterSpacing: letterSpacingBody2, fontSize: sizeBody2);

    styleButton = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightButton, fontStyle: fontStyleButton, letterSpacing: letterSpacingButton, fontSize: sizeButton);

    styleCaption = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightCaption, fontStyle: fontStyleCaption, letterSpacing: letterSpacingCaption, fontSize: sizeCaption);

    styleOverline = baseStyle.copyWith(color: colors.textPrimary, fontWeight: fontWeightOverline, fontStyle: fontStyleOverline, letterSpacing: letterSpacingOverline, fontSize: sizeOverline);
    

  }


}