import 'package:flutter/widgets.dart';
import 'package:flutterskel/configuration/ConfigurationRepository.dart';
import 'package:flutterskel/datasource/DataSource.dart';
import 'package:flutterskel/resources/images/ImageResources.dart';
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart';
import 'package:flutterskel/routing/page/PageRouter.dart';
import 'package:flutterskel/routing/url/UrlRouter.dart';
import 'package:flutterskel/userinformation/UserInformationRepository.dart';
import 'package:flutterskel/resources/icons/IconResources.dart';
import 'package:provider/provider.dart';

/// Raw abstract State class
/// This class should only be extended by an abstract class within the app, which fills in all the info except [StateWidget]
/// [StateWidget] should be filled in by the final StatefulWidget class
abstract class RawBaseStatelessWidget<Data extends DataSource,
                                      URouter extends UrlRouter,
                                      PRouter extends PageRouter,
                                      Images extends ImageResources,
                                      ConfigRepo extends ConfigurationRepository,
                                      UserRepo extends UserInformationRepository,
                                      StringRepo extends StringsRepository,
                                      ThemeRepo extends ThemeRepository,
                                      Icons extends IconResources> extends StatelessWidget {

  Data getDataSource(BuildContext context) => Provider.of<Data>(context);

  URouter urlRouter(BuildContext context) => Provider.of<URouter>(context);

  PRouter pageRouter(BuildContext context) => Provider.of<PRouter>(context);

  Images images(BuildContext context) => Provider.of<Images>(context);

  ConfigRepo configuration(BuildContext context) => Provider.of<ConfigRepo>(context);

  UserRepo userInformation(BuildContext context) => Provider.of<UserRepo>(context);

  StringRepo strings(BuildContext context) => Provider.of<StringRepo>(context);

  ThemeRepo themes(BuildContext context) => Provider.of<ThemeRepo>(context);

  Icons icons(BuildContext context) => Provider.of<Icons>(context);

}
